@@ -24,7 +24,7 @@ use tracing::debug;
 #[derive(Debug, Copy, Clone)]
 struct Context {
     /// The scope that contains any new variables declared.
-    var_parent: Option<Scope>,
+    var_parent: (Option<Scope>, ScopeCompatibility),
 
     /// Region parent of expressions, etc.
     parent: Option<Scope>,
@@ -38,12 +38,26 @@ struct ScopeResolutionVisitor<'tcx> {
 
     cx: Context,
 
-    extended_super_lets: FxHashMap<hir::ItemLocalId, Option<Scope>>,
+    extended_super_lets: FxHashMap<hir::ItemLocalId, ExtendedTemporaryScope>,
+}
+
+#[derive(Copy, Clone)]
+struct ExtendedTemporaryScope {
+    /// The scope of extended temporaries.
+    scope: Option<Scope>,
+    /// Whether this lifetime originated from a regular `let` or a `super let` initializer. In the
+    /// latter case, this scope may shorten after #145838 if applied to temporaries within block
+    /// tail expressions.
+    let_kind: LetKind,
+    /// Whether this scope will shorten after #145838. If this is applied to a temporary value,
+    /// we'll emit the `macro_extended_temporary_scopes` lint.
+    compat: ScopeCompatibility,
 }
 
 /// Records the lifetime of a local variable as `cx.var_parent`
 fn record_var_lifetime(visitor: &mut ScopeResolutionVisitor<'_>, var_id: hir::ItemLocalId) {
-    match visitor.cx.var_parent {
+    let (var_parent_scope, var_parent_compat) = visitor.cx.var_parent;
+    match var_parent_scope {
         None => {
             // this can happen in extern fn declarations like
             //
@@ -51,6 +65,9 @@ fn record_var_lifetime(visitor: &mut ScopeResolutionVisitor<'_>, var_id: hir::It
         }
         Some(parent_scope) => visitor.scope_tree.record_var_scope(var_id, parent_scope),
     }
+    if let ScopeCompatibility::FutureIncompatible { shortens_to } = var_parent_compat {
+        visitor.scope_tree.record_future_incompatible_var_scope(var_id, shortens_to);
+    }
 }
 
 fn resolve_block<'tcx>(
@@ -88,7 +105,7 @@ fn resolve_block<'tcx>(
     // itself has returned.
 
     visitor.enter_node_scope_with_dtor(blk.hir_id.local_id, terminating);
-    visitor.cx.var_parent = visitor.cx.parent;
+    visitor.cx.var_parent = (visitor.cx.parent, ScopeCompatibility::FutureCompatible);
 
     {
         // This block should be kept approximately in sync with
@@ -107,7 +124,8 @@ fn resolve_block<'tcx>(
                         local_id: blk.hir_id.local_id,
                         data: ScopeData::Remainder(FirstStatementIndex::new(i)),
                     });
-                    visitor.cx.var_parent = visitor.cx.parent;
+                    visitor.cx.var_parent =
+                        (visitor.cx.parent, ScopeCompatibility::FutureCompatible);
                     visitor.visit_stmt(statement);
                     // We need to back out temporarily to the last enclosing scope
                     // for the `else` block, so that even the temporaries receiving
@@ -131,7 +149,8 @@ fn resolve_block<'tcx>(
                         local_id: blk.hir_id.local_id,
                         data: ScopeData::Remainder(FirstStatementIndex::new(i)),
                     });
-                    visitor.cx.var_parent = visitor.cx.parent;
+                    visitor.cx.var_parent =
+                        (visitor.cx.parent, ScopeCompatibility::FutureCompatible);
                     visitor.visit_stmt(statement)
                 }
                 hir::StmtKind::Item(..) => {
@@ -195,10 +214,15 @@ fn resolve_arm<'tcx>(visitor: &mut ScopeResolutionVisitor<'tcx>, arm: &'tcx hir:
     let prev_cx = visitor.cx;
 
     visitor.enter_node_scope_with_dtor(arm.hir_id.local_id, true);
-    visitor.cx.var_parent = visitor.cx.parent;
+    visitor.cx.var_parent = (visitor.cx.parent, ScopeCompatibility::FutureCompatible);
 
     resolve_pat(visitor, arm.pat);
     if let Some(guard) = arm.guard {
+        // We introduce a new scope to contain bindings and temporaries from `if let` guards, to
+        // ensure they're dropped before the arm's pattern's bindings. This extends to the end of
+        // the arm body and is the scope of its locals as well.
+        visitor.enter_scope(Scope { local_id: arm.hir_id.local_id, data: ScopeData::MatchGuard });
+        visitor.cx.var_parent = (visitor.cx.parent, ScopeCompatibility::FutureCompatible);
         resolve_cond(visitor, guard);
     }
     resolve_expr(visitor, arm.body, false);
@@ -355,7 +379,7 @@ fn resolve_expr<'tcx>(
                 ScopeData::IfThen
             };
             visitor.enter_scope(Scope { local_id: then.hir_id.local_id, data });
-            visitor.cx.var_parent = visitor.cx.parent;
+            visitor.cx.var_parent = (visitor.cx.parent, ScopeCompatibility::FutureCompatible);
             resolve_cond(visitor, cond);
             resolve_expr(visitor, then, true);
             visitor.cx = expr_cx;
@@ -370,7 +394,7 @@ fn resolve_expr<'tcx>(
                 ScopeData::IfThen
             };
             visitor.enter_scope(Scope { local_id: then.hir_id.local_id, data });
-            visitor.cx.var_parent = visitor.cx.parent;
+            visitor.cx.var_parent = (visitor.cx.parent, ScopeCompatibility::FutureCompatible);
             resolve_cond(visitor, cond);
             resolve_expr(visitor, then, true);
             visitor.cx = expr_cx;
@@ -462,41 +486,57 @@ fn resolve_local<'tcx>(
     // A, but the inner rvalues `a()` and `b()` have an extended lifetime
     // due to rule C.
 
-    if let_kind == LetKind::Super {
-        if let Some(scope) = visitor.extended_super_lets.remove(&pat.unwrap().hir_id.local_id) {
-            // This expression was lifetime-extended by a parent let binding. E.g.
-            //
-            //     let a = {
-            //         super let b = temp();
-            //         &b
-            //     };
-            //
-            // (Which needs to behave exactly as: let a = &temp();)
-            //
-            // Processing of `let a` will have already decided to extend the lifetime of this
-            // `super let` to its own var_scope. We use that scope.
-            visitor.cx.var_parent = scope;
-        } else {
-            // This `super let` is not subject to lifetime extension from a parent let binding. E.g.
-            //
-            //     identity({ super let x = temp(); &x }).method();
-            //
-            // (Which needs to behave exactly as: identity(&temp()).method();)
-            //
-            // Iterate up to the enclosing destruction scope to find the same scope that will also
-            // be used for the result of the block itself.
-            while let Some(s) = visitor.cx.var_parent {
-                let parent = visitor.scope_tree.parent_map.get(&s).cloned();
-                if let Some(Scope { data: ScopeData::Destruction, .. }) = parent {
-                    break;
+    let (source_let_kind, compat) = match let_kind {
+        // Normal `let` initializers are unaffected by #145838.
+        LetKind::Regular => {
+            (LetKind::Regular, ScopeCompatibility::FutureCompatible)
+        }
+        LetKind::Super => {
+            if let Some(scope) = visitor.extended_super_lets.remove(&pat.unwrap().hir_id.local_id) {
+                // This expression was lifetime-extended by a parent let binding. E.g.
+                //
+                //     let a = {
+                //         super let b = temp();
+                //         &b
+                //     };
+                //
+                // (Which needs to behave exactly as: let a = &temp();)
+                //
+                // Processing of `let a` will have already decided to extend the lifetime of this
+                // `super let` to its own var_scope. We use that scope.
+                visitor.cx.var_parent = (scope.scope, scope.compat);
+                // Inherit compatibility from the original `let` statement. If the original `let`
+                // was regular, lifetime extension should apply as normal. If the original `let` was
+                // `super`, blocks within the initializer will be affected by #145838.
+                (scope.let_kind, scope.compat)
+            } else {
+                // This `super let` is not subject to lifetime extension from a parent let binding. E.g.
+                //
+                //     identity({ super let x = temp(); &x }).method();
+                //
+                // (Which needs to behave exactly as: identity(&temp()).method();)
+                //
+                // Iterate up to the enclosing destruction scope to find the same scope that will also
+                // be used for the result of the block itself.
+                if let (Some(inner_scope), _) = visitor.cx.var_parent {
+                    // NB(@dianne): This could use the incompatibility reported by
+                    // `default_temporary_scope` to make `tail_expr_drop_order` more comprehensive.
+                    visitor.cx.var_parent =
+                        (visitor.scope_tree.default_temporary_scope(inner_scope).0, ScopeCompatibility::FutureCompatible);
                 }
-                visitor.cx.var_parent = parent;
+                // Blocks within the initializer will be affected by #145838.
+                (LetKind::Super, ScopeCompatibility::FutureCompatible)
             }
         }
-    }
+    };
 
     if let Some(expr) = init {
-        record_rvalue_scope_if_borrow_expr(visitor, expr, visitor.cx.var_parent);
+        let scope = ExtendedTemporaryScope {
+            scope: visitor.cx.var_parent.0,
+            let_kind: source_let_kind,
+            compat,
+        };
+        record_rvalue_scope_if_borrow_expr(visitor, expr, scope);
 
         if let Some(pat) = pat {
             if is_binding_pat(pat) {
@@ -504,7 +544,8 @@ fn resolve_local<'tcx>(
                     expr.hir_id,
                     RvalueCandidate {
                         target: expr.hir_id.local_id,
-                        lifetime: visitor.cx.var_parent,
+                        lifetime: visitor.cx.var_parent.0,
+                        compat: visitor.cx.var_parent.1,
                     },
                 );
             }
@@ -606,50 +647,106 @@ fn resolve_local<'tcx>(
     fn record_rvalue_scope_if_borrow_expr<'tcx>(
         visitor: &mut ScopeResolutionVisitor<'tcx>,
         expr: &hir::Expr<'_>,
-        blk_id: Option<Scope>,
+        scope: ExtendedTemporaryScope,
     ) {
         match expr.kind {
             hir::ExprKind::AddrOf(_, _, subexpr) => {
-                record_rvalue_scope_if_borrow_expr(visitor, subexpr, blk_id);
+                record_rvalue_scope_if_borrow_expr(visitor, subexpr, scope);
                 visitor.scope_tree.record_rvalue_candidate(
                     subexpr.hir_id,
-                    RvalueCandidate { target: subexpr.hir_id.local_id, lifetime: blk_id },
+                    RvalueCandidate {
+                        target: subexpr.hir_id.local_id,
+                        lifetime: scope.scope,
+                        compat: scope.compat,
+                    },
                 );
             }
             hir::ExprKind::Struct(_, fields, _) => {
                 for field in fields {
-                    record_rvalue_scope_if_borrow_expr(visitor, field.expr, blk_id);
+                    record_rvalue_scope_if_borrow_expr(visitor, field.expr, scope);
                 }
             }
             hir::ExprKind::Array(subexprs) | hir::ExprKind::Tup(subexprs) => {
                 for subexpr in subexprs {
-                    record_rvalue_scope_if_borrow_expr(visitor, subexpr, blk_id);
+                    record_rvalue_scope_if_borrow_expr(visitor, subexpr, scope);
                 }
             }
             hir::ExprKind::Cast(subexpr, _) => {
-                record_rvalue_scope_if_borrow_expr(visitor, subexpr, blk_id)
+                record_rvalue_scope_if_borrow_expr(visitor, subexpr, scope)
             }
             hir::ExprKind::Block(block, _) => {
                 if let Some(subexpr) = block.expr {
-                    record_rvalue_scope_if_borrow_expr(visitor, subexpr, blk_id);
+                    let tail_expr_scope =
+                        if scope.let_kind == LetKind::Super && block.span.at_least_rust_2024() {
+                            // The tail expression will no longer be extending after #145838.
+                            // Since tail expressions are temporary scopes in Rust 2024, lint on
+                            // temporaries that acquire this (longer) lifetime.
+                            ExtendedTemporaryScope {
+                                compat: ScopeCompatibility::FutureIncompatible {
+                                    shortens_to: Scope {
+                                        local_id: subexpr.hir_id.local_id,
+                                        data: ScopeData::Node,
+                                    },
+                                },
+                                ..scope
+                            }
+                        } else {
+                            // This is extended by a regular `let`, so it won't be changed.
+                            scope
+                        };
+                    record_rvalue_scope_if_borrow_expr(visitor, subexpr, tail_expr_scope);
                 }
                 for stmt in block.stmts {
                     if let hir::StmtKind::Let(local) = stmt.kind
                         && let Some(_) = local.super_
                     {
-                        visitor.extended_super_lets.insert(local.pat.hir_id.local_id, blk_id);
+                        visitor.extended_super_lets.insert(local.pat.hir_id.local_id, scope);
                     }
                 }
             }
             hir::ExprKind::If(_, then_block, else_block) => {
-                record_rvalue_scope_if_borrow_expr(visitor, then_block, blk_id);
+                let then_scope = if scope.let_kind == LetKind::Super {
+                    // The then and else blocks will no longer be extending after #145838.
+                    // Since `if` blocks are temporary scopes in all editions, lint on temporaries
+                    // that acquire this (longer) lifetime.
+                    ExtendedTemporaryScope {
+                        compat: ScopeCompatibility::FutureIncompatible {
+                            shortens_to: Scope {
+                                local_id: then_block.hir_id.local_id,
+                                data: ScopeData::Node,
+                            },
+                        },
+                        ..scope
+                    }
+                } else {
+                    // This is extended by a regular `let`, so it won't be changed.
+                    scope
+                };
+                record_rvalue_scope_if_borrow_expr(visitor, then_block, then_scope);
                 if let Some(else_block) = else_block {
-                    record_rvalue_scope_if_borrow_expr(visitor, else_block, blk_id);
+                    let else_scope = if scope.let_kind == LetKind::Super {
+                        // The then and else blocks will no longer be extending after #145838.
+                        // Since `if` blocks are temporary scopes in all editions, lint on temporaries
+                        // that acquire this (longer) lifetime.
+                        ExtendedTemporaryScope {
+                            compat: ScopeCompatibility::FutureIncompatible {
+                                shortens_to: Scope {
+                                    local_id: else_block.hir_id.local_id,
+                                    data: ScopeData::Node,
+                                },
+                            },
+                            ..scope
+                        }
+                    } else {
+                        // This is extended by a regular `let`, so it won't be changed.
+                        scope
+                    };
+                    record_rvalue_scope_if_borrow_expr(visitor, else_block, else_scope);
                 }
             }
             hir::ExprKind::Match(_, arms, _) => {
                 for arm in arms {
-                    record_rvalue_scope_if_borrow_expr(visitor, arm.body, blk_id);
+                    record_rvalue_scope_if_borrow_expr(visitor, arm.body, scope);
                 }
             }
             hir::ExprKind::Call(func, args) => {
@@ -662,7 +759,7 @@ fn resolve_local<'tcx>(
                     && let Res::SelfCtor(_) | Res::Def(DefKind::Ctor(_, CtorKind::Fn), _) = path.res
                 {
                     for arg in args {
-                        record_rvalue_scope_if_borrow_expr(visitor, arg, blk_id);
+                        record_rvalue_scope_if_borrow_expr(visitor, arg, scope);
                     }
                 }
             }
@@ -729,7 +826,7 @@ impl<'tcx> Visitor<'tcx> for ScopeResolutionVisitor<'tcx> {
         self.enter_body(body.value.hir_id, |this| {
             if this.tcx.hir_body_owner_kind(owner_id).is_fn_or_closure() {
                 // The arguments and `self` are parented to the fn.
-                this.cx.var_parent = this.cx.parent;
+                this.cx.var_parent = (this.cx.parent, ScopeCompatibility::FutureCompatible);
                 for param in body.params {
                     this.visit_pat(param.pat);
                 }
@@ -755,7 +852,7 @@ impl<'tcx> Visitor<'tcx> for ScopeResolutionVisitor<'tcx> {
                 // would *not* let the `f()` temporary escape into an outer scope
                 // (i.e., `'static`), which means that after `g` returns, it drops,
                 // and all the associated destruction scope rules apply.
-                this.cx.var_parent = None;
+                this.cx.var_parent = (None, ScopeCompatibility::FutureCompatible);
                 this.enter_scope(Scope {
                     local_id: body.value.hir_id.local_id,
                     data: ScopeData::Destruction,
@@ -807,7 +904,7 @@ pub(crate) fn region_scope_tree(tcx: TyCtxt<'_>, def_id: DefId) -> &ScopeTree {
         let mut visitor = ScopeResolutionVisitor {
             tcx,
             scope_tree: ScopeTree::default(),
-            cx: Context { parent: None, var_parent: None },
+            cx: Context { parent: None, var_parent: (None, ScopeCompatibility::FutureCompatible) },
             extended_super_lets: Default::default(),
         };
 
