@@ -2,7 +2,9 @@ use hir::Node;
 use hir::def_id::DefId;
 use rustc_hir as hir;
 use rustc_middle::bug;
-use rustc_middle::middle::region::{RvalueCandidate, Scope, ScopeTree};
+use rustc_middle::middle::region::{
+    ScopeCompatibility, RvalueCandidate, Scope, ScopeTree,
+};
 use rustc_middle::ty::RvalueScopes;
 use tracing::debug;
 
@@ -29,6 +31,7 @@ fn record_rvalue_scope_rec(
     rvalue_scopes: &mut RvalueScopes,
     mut expr: &hir::Expr<'_>,
     lifetime: Option<Scope>,
+    compat: ScopeCompatibility,
 ) {
     loop {
         // Note: give all the expressions matching `ET` with the
@@ -37,7 +40,7 @@ fn record_rvalue_scope_rec(
         // into a temporary, we request the temporary scope of the
         // outer expression.
 
-        rvalue_scopes.record_rvalue_scope(expr.hir_id.local_id, lifetime);
+        rvalue_scopes.record_rvalue_scope(expr.hir_id.local_id, lifetime, compat);
 
         match expr.kind {
             hir::ExprKind::AddrOf(_, _, subexpr)
@@ -58,7 +61,7 @@ fn record_rvalue_scope(
     candidate: &RvalueCandidate,
 ) {
     debug!("resolve_rvalue_scope(expr={expr:?}, candidate={candidate:?})");
-    record_rvalue_scope_rec(rvalue_scopes, expr, candidate.lifetime)
+    record_rvalue_scope_rec(rvalue_scopes, expr, candidate.lifetime, candidate.compat)
     // FIXME(@dingxiangfei2009): handle the candidates in the function call arguments
 }
 
