@@ -5,6 +5,6 @@ fn main() {
     let r1 = &mut s;
     let r2 = &mut s;
 
-    println!("{}, {}", r1, r2);
+    println!("{r1}, {r2}");
     // ANCHOR_END: here
 }
