@@ -1,16 +1,21 @@
-#![allow(unused_variables)]
-fn main() {}
+pub fn add(left: u64, right: u64) -> u64 {
+    left + right
+}
 
-// ANCHOR: here
 #[cfg(test)]
 mod tests {
+    use super::*;
+
+    // ANCHOR: here
     #[test]
     fn it_works() -> Result<(), String> {
-        if 2 + 2 == 4 {
+        let result = add(2, 2);
+
+        if result == 4 {
             Ok(())
         } else {
             Err(String::from("two plus two does not equal four"))
         }
     }
+    // ANCHOR_END: here
 }
-// ANCHOR_END: here
