@@ -3,7 +3,7 @@ use hello::ThreadPool;
 // ANCHOR_END: here
 use std::{
     fs,
-    io::{prelude::*, BufReader},
+    io::{BufReader, prelude::*},
     net::{TcpListener, TcpStream},
     thread,
     time::Duration,
