@@ -11,8 +11,8 @@ use std::io::Seek;
 use std::io::SeekFrom;
 use std::time::Duration;
 
-use anyhow::bail;
 use anyhow::Context as _;
+use anyhow::bail;
 use cargo_credential::Operation;
 use cargo_credential::Secret;
 use cargo_util::paths;
@@ -21,35 +21,36 @@ use crates_io::NewCrateDependency;
 use crates_io::Registry;
 use itertools::Itertools;
 
-use crate::core::dependency::DepKind;
-use crate::core::manifest::ManifestMetadata;
-use crate::core::resolver::CliFeatures;
+use crate::CargoResult;
+use crate::GlobalContext;
 use crate::core::Dependency;
 use crate::core::Package;
 use crate::core::PackageId;
 use crate::core::PackageIdSpecQuery;
 use crate::core::SourceId;
 use crate::core::Workspace;
+use crate::core::dependency::DepKind;
+use crate::core::manifest::ManifestMetadata;
+use crate::core::resolver::CliFeatures;
 use crate::ops;
-use crate::ops::registry::RegistrySourceIds;
 use crate::ops::PackageOpts;
 use crate::ops::Packages;
 use crate::ops::RegistryOrIndex;
-use crate::sources::source::QueryKind;
-use crate::sources::source::Source;
+use crate::ops::registry::RegistrySourceIds;
+use crate::sources::CRATES_IO_REGISTRY;
 use crate::sources::RegistrySource;
 use crate::sources::SourceConfigMap;
-use crate::sources::CRATES_IO_REGISTRY;
-use crate::util::auth;
-use crate::util::cache_lock::CacheLockMode;
-use crate::util::context::JobsConfig;
-use crate::util::toml::prepare_for_publish;
+use crate::sources::source::QueryKind;
+use crate::sources::source::Source;
 use crate::util::Graph;
 use crate::util::Progress;
 use crate::util::ProgressStyle;
 use crate::util::VersionExt as _;
-use crate::CargoResult;
-use crate::GlobalContext;
+use crate::util::auth;
+use crate::util::cache_lock::CacheLockMode;
+use crate::util::context::JobsConfig;
+use crate::util::errors::ManifestError;
+use crate::util::toml::prepare_for_publish;
 
 use super::super::check_dep_has_version;
 
@@ -68,18 +69,8 @@ pub struct PublishOpts<'gctx> {
 }
 
 pub fn publish(ws: &Workspace<'_>, opts: &PublishOpts<'_>) -> CargoResult<()> {
-    let multi_package_mode = ws.gctx().cli_unstable().package_workspace;
     let specs = opts.to_publish.to_package_id_specs(ws)?;
 
-    if !multi_package_mode {
-        if specs.len() > 1 {
-            bail!("the `-p` argument must be specified to select a single package to publish")
-        }
-        if Packages::Default == opts.to_publish && ws.is_virtual() {
-            bail!("the `-p` argument must be specified in the root of a virtual workspace")
-        }
-    }
-
     let member_ids: Vec<_> = ws.members().map(|p| p.package_id()).collect();
     // Check that the specs match members.
     for spec in &specs {
@@ -88,10 +79,46 @@ pub fn publish(ws: &Workspace<'_>, opts: &PublishOpts<'_>) -> CargoResult<()> {
     let mut pkgs = ws.members_with_features(&specs, &opts.cli_features)?;
     // In `members_with_features_old`, it will add "current" package (determined by the cwd)
     // So we need filter
-    pkgs = pkgs
+    pkgs.retain(|(m, _)| specs.iter().any(|spec| spec.matches(m.package_id())));
+
+    let (unpublishable, pkgs): (Vec<_>, Vec<_>) = pkgs
         .into_iter()
-        .filter(|(m, _)| specs.iter().any(|spec| spec.matches(m.package_id())))
-        .collect();
+        .partition(|(pkg, _)| pkg.publish() == &Some(vec![]));
+    // If `--workspace` is passed,
+    // the intent is more like "publish all publisable packages in this workspace",
+    // so skip `publish=false` packages.
+    let allow_unpublishable = match &opts.to_publish {
+        Packages::Default => ws.is_virtual(),
+        Packages::All(_) => true,
+        Packages::OptOut(_) => true,
+        Packages::Packages(_) => false,
+    };
+    if !unpublishable.is_empty() && !allow_unpublishable {
+        bail!(
+            "{} cannot be published.\n\
+            `package.publish` must be set to `true` or a non-empty list in Cargo.toml to publish.",
+            unpublishable
+                .iter()
+                .map(|(pkg, _)| format!("`{}`", pkg.name()))
+                .join(", "),
+        );
+    }
+
+    if pkgs.is_empty() {
+        if allow_unpublishable {
+            let n = unpublishable.len();
+            let plural = if n == 1 { "" } else { "s" };
+            ws.gctx().shell().warn(format_args!(
+                "nothing to publish, but found {n} unpublishable package{plural}"
+            ))?;
+            ws.gctx().shell().note(format_args!(
+                "to publish packages, set `package.publish` to `true` or a non-empty list"
+            ))?;
+            return Ok(());
+        } else {
+            unreachable!("must have at least one publishable package");
+        }
+    }
 
     let just_pkgs: Vec<_> = pkgs.iter().map(|p| p.0).collect();
     let reg_or_index = match opts.reg_or_index.clone() {
@@ -102,7 +129,7 @@ pub fn publish(ws: &Workspace<'_>, opts: &PublishOpts<'_>) -> CargoResult<()> {
         None => {
             let reg = super::infer_registry(&just_pkgs)?;
             validate_registry(&just_pkgs, reg.as_ref())?;
-            if let Some(RegistryOrIndex::Registry(ref registry)) = &reg {
+            if let Some(RegistryOrIndex::Registry(registry)) = &reg {
                 if registry != CRATES_IO_REGISTRY {
                     // Don't warn for crates.io.
                     opts.gctx.shell().note(&format!(
@@ -134,7 +161,15 @@ pub fn publish(ws: &Workspace<'_>, opts: &PublishOpts<'_>) -> CargoResult<()> {
 
         for (pkg, _) in &pkgs {
             verify_unpublished(pkg, &mut source, &source_ids, opts.dry_run, opts.gctx)?;
-            verify_dependencies(pkg, &registry, source_ids.original)?;
+            verify_dependencies(pkg, &registry, source_ids.original).map_err(|err| {
+                ManifestError::new(
+                    err.context(format!(
+                        "failed to verify manifest at `{}`",
+                        pkg.manifest_path().display()
+                    )),
+                    pkg.manifest_path().into(),
+                )
+            })?;
         }
     }
 
@@ -144,8 +179,10 @@ pub fn publish(ws: &Workspace<'_>, opts: &PublishOpts<'_>) -> CargoResult<()> {
             gctx: opts.gctx,
             verify: opts.verify,
             list: false,
+            fmt: ops::PackageMessageFormat::Human,
             check_metadata: true,
             allow_dirty: opts.allow_dirty,
+            include_lockfile: true,
             // `package_with_dep_graph` ignores this field in favor of
             // the already-resolved list of packages
             to_package: ops::Packages::Default,
@@ -154,6 +191,7 @@ pub fn publish(ws: &Workspace<'_>, opts: &PublishOpts<'_>) -> CargoResult<()> {
             keep_going: opts.keep_going,
             cli_features: opts.cli_features.clone(),
             reg_or_index: reg_or_index.clone(),
+            dry_run: opts.dry_run,
         },
         pkgs,
     )?;
@@ -172,7 +210,8 @@ pub fn publish(ws: &Workspace<'_>, opts: &PublishOpts<'_>) -> CargoResult<()> {
         // `b`, and we uploaded `a` and `b` but only confirmed `a`, then on
         // the following pass through the outer loop nothing will be ready for
         // upload.
-        for pkg_id in plan.take_ready() {
+        let mut ready = plan.take_ready();
+        while let Some(pkg_id) = ready.pop_first() {
             let (pkg, (_features, tarball)) = &pkg_dep_graph.packages[&pkg_id];
             opts.gctx.shell().status("Uploading", pkg.package_id())?;
 
@@ -198,6 +237,19 @@ pub fn publish(ws: &Workspace<'_>, opts: &PublishOpts<'_>) -> CargoResult<()> {
                 )?));
             }
 
+            let workspace_context = || {
+                let mut remaining = ready.clone();
+                remaining.extend(plan.iter());
+                if !remaining.is_empty() {
+                    format!(
+                        "\n\nnote: the following crates have not been published yet:\n  {}",
+                        remaining.into_iter().join("\n  ")
+                    )
+                } else {
+                    String::new()
+                }
+            };
+
             transmit(
                 opts.gctx,
                 ws,
@@ -206,6 +258,7 @@ pub fn publish(ws: &Workspace<'_>, opts: &PublishOpts<'_>) -> CargoResult<()> {
                 &mut registry,
                 source_ids.original,
                 opts.dry_run,
+                workspace_context,
             )?;
             to_confirm.insert(pkg_id);
 
@@ -231,13 +284,52 @@ pub fn publish(ws: &Workspace<'_>, opts: &PublishOpts<'_>) -> CargoResult<()> {
                 DEFAULT_TIMEOUT
             };
             if 0 < timeout {
+                let source_description = source.source_id().to_string();
+                let short_pkg_descriptions = package_list(to_confirm.iter().copied(), "or");
+                if plan.is_empty() {
+                    opts.gctx.shell().note(format!(
+                    "waiting for {short_pkg_descriptions} to be available at {source_description}.\n\
+                    You may press ctrl-c to skip waiting; the {crate} should be available shortly.",
+                    crate = if to_confirm.len() == 1 { "crate" } else {"crates"}
+                ))?;
+                } else {
+                    opts.gctx.shell().note(format!(
+                    "waiting for {short_pkg_descriptions} to be available at {source_description}.\n\
+                    {count} remaining {crate} to be published",
+                    count = plan.len(),
+                    crate = if plan.len() == 1 { "crate" } else {"crates"}
+                ))?;
+                }
+
                 let timeout = Duration::from_secs(timeout);
-                wait_for_any_publish_confirmation(
+                let confirmed = wait_for_any_publish_confirmation(
                     opts.gctx,
                     source_ids.original,
                     &to_confirm,
                     timeout,
-                )?
+                )?;
+                if !confirmed.is_empty() {
+                    let short_pkg_description = package_list(confirmed.iter().copied(), "and");
+                    opts.gctx.shell().status(
+                        "Published",
+                        format!("{short_pkg_description} at {source_description}"),
+                    )?;
+                } else {
+                    let short_pkg_descriptions = package_list(to_confirm.iter().copied(), "or");
+                    opts.gctx.shell().warn(format!(
+                        "timed out waiting for {short_pkg_descriptions} to be available in {source_description}",
+                    ))?;
+                    opts.gctx.shell().note(format!(
+                        "the registry may have a backlog that is delaying making the \
+                        {crate} available. The {crate} should be available soon.",
+                        crate = if to_confirm.len() == 1 {
+                            "crate"
+                        } else {
+                            "crates"
+                        }
+                    ))?;
+                }
+                confirmed
             } else {
                 BTreeSet::new()
             }
@@ -251,7 +343,9 @@ pub fn publish(ws: &Workspace<'_>, opts: &PublishOpts<'_>) -> CargoResult<()> {
                 break;
             } else {
                 let failed_list = package_list(plan.iter(), "and");
-                bail!("unable to publish {failed_list} due to time out while waiting for published dependencies to be available.");
+                bail!(
+                    "unable to publish {failed_list} due to a timeout while waiting for published dependencies to be available."
+                );
             }
         }
         for id in &confirmed {
@@ -278,17 +372,10 @@ fn wait_for_any_publish_confirmation(
     // of independent progress bars can be a little confusing. There is an
     // overall progress bar managed here.
     source.set_quiet(true);
-    let source_description = source.source_id().to_string();
 
     let now = std::time::Instant::now();
     let sleep_time = Duration::from_secs(1);
     let max = timeout.as_secs() as usize;
-    // Short does not include the registry name.
-    let short_pkg_descriptions = package_list(pkgs.iter().copied(), "or");
-    gctx.shell().note(format!(
-        "waiting for {short_pkg_descriptions} to be available at {source_description}.\n\
-        You may press ctrl-c to skip waiting; the crate should be available shortly."
-    ))?;
     let mut progress = Progress::with_style("Waiting", ProgressStyle::Ratio, gctx);
     progress.tick_now(0, max, "")?;
     let available = loop {
@@ -317,30 +404,12 @@ fn wait_for_any_publish_confirmation(
 
         let elapsed = now.elapsed();
         if timeout < elapsed {
-            gctx.shell().warn(format!(
-                "timed out waiting for {short_pkg_descriptions} to be available in {source_description}",
-            ))?;
-            gctx.shell().note(
-                "the registry may have a backlog that is delaying making the \
-                crate available. The crate should be available soon.",
-            )?;
             break BTreeSet::new();
         }
 
         progress.tick_now(elapsed.as_secs() as usize, max, "")?;
         std::thread::sleep(sleep_time);
     };
-    if !available.is_empty() {
-        let short_pkg_description = available
-            .iter()
-            .map(|pkg| format!("{} v{}", pkg.name(), pkg.version()))
-            .sorted()
-            .join(", ");
-        gctx.shell().status(
-            "Published",
-            format!("{short_pkg_description} at {source_description}"),
-        )?;
-    }
 
     Ok(available)
 }
@@ -431,12 +500,14 @@ fn verify_dependencies(
             // but also prevents someone using `--index` to specify
             // something that points to crates.io.
             if registry_src.is_crates_io() || registry.host_is_crates_io() {
-                bail!("crates cannot be published to crates.io with dependencies sourced from other\n\
+                bail!(
+                    "crates cannot be published to crates.io with dependencies sourced from other\n\
                        registries. `{}` needs to be published to crates.io before publishing this crate.\n\
                        (crate `{}` is pulled from {})",
-                      dep.package_name(),
-                      dep.package_name(),
-                      dep.source_id());
+                    dep.package_name(),
+                    dep.package_name(),
+                    dep.source_id()
+                );
             }
         }
     }
@@ -576,6 +647,7 @@ fn transmit(
     registry: &mut Registry,
     registry_id: SourceId,
     dry_run: bool,
+    workspace_context: impl Fn() -> String,
 ) -> CargoResult<()> {
     let new_crate = prepare_transmit(gctx, ws, pkg, registry_id)?;
 
@@ -585,9 +657,15 @@ fn transmit(
         return Ok(());
     }
 
-    let warnings = registry
-        .publish(&new_crate, tarball)
-        .with_context(|| format!("failed to publish to registry at {}", registry.host()))?;
+    let warnings = registry.publish(&new_crate, tarball).with_context(|| {
+        format!(
+            "failed to publish {} v{} to registry at {}{}",
+            pkg.name(),
+            pkg.version(),
+            registry.host(),
+            workspace_context()
+        )
+    })?;
 
     if !warnings.invalid_categories.is_empty() {
         let msg = format!(
@@ -652,6 +730,10 @@ impl PublishPlan {
         self.dependencies_count.is_empty()
     }
 
+    fn len(&self) -> usize {
+        self.dependencies_count.len()
+    }
+
     /// Returns the set of packages that are ready for publishing (i.e. have no outstanding dependencies).
     ///
     /// These will not be returned in future calls.
@@ -688,7 +770,7 @@ impl PublishPlan {
 fn package_list(pkgs: impl IntoIterator<Item = PackageId>, final_sep: &str) -> String {
     let mut names: Vec<_> = pkgs
         .into_iter()
-        .map(|pkg| format!("`{} v{}`", pkg.name(), pkg.version()))
+        .map(|pkg| format!("{} v{}", pkg.name(), pkg.version()))
         .collect();
     names.sort();
 
@@ -703,16 +785,6 @@ fn package_list(pkgs: impl IntoIterator<Item = PackageId>, final_sep: &str) -> S
 }
 
 fn validate_registry(pkgs: &[&Package], reg_or_index: Option<&RegistryOrIndex>) -> CargoResult<()> {
-    for pkg in pkgs {
-        if pkg.publish() == &Some(Vec::new()) {
-            bail!(
-                    "`{}` cannot be published.\n\
-                    `package.publish` must be set to `true` or a non-empty list in Cargo.toml to publish.",
-                    pkg.name(),
-                );
-        }
-    }
-
     let reg_name = match reg_or_index {
         Some(RegistryOrIndex::Registry(r)) => Some(r.as_str()),
         None => Some(CRATES_IO_REGISTRY),
