@@ -1,8 +1,11 @@
-use cargo_test_support::prelude::*;
+use crate::prelude::*;
 use cargo_test_support::{file, registry::RegistryBuilder};
 
+use super::init_registry_without_token;
+
 #[cargo_test]
 fn case() {
+    init_registry_without_token();
     let _ = RegistryBuilder::new()
         .alternative()
         .no_configure_token()
