@@ -1,8 +1,8 @@
+use crate::prelude::*;
+use cargo_test_support::Project;
 use cargo_test_support::current_dir;
 use cargo_test_support::file;
-use cargo_test_support::prelude::*;
 use cargo_test_support::str;
-use cargo_test_support::Project;
 
 #[cargo_test]
 fn case() {
