@@ -2,13 +2,15 @@
 
 use std::fs::File;
 
-use cargo_test_support::cross_compile::{self, alternate};
+use crate::prelude::*;
+use crate::utils::cargo_process;
+use crate::utils::cross_compile::disabled as cross_compile_disabled;
+use cargo_test_support::cross_compile::alternate;
 use cargo_test_support::paths;
-use cargo_test_support::prelude::*;
 use cargo_test_support::publish::validate_crate_contents;
 use cargo_test_support::registry::{Dependency, Package};
 use cargo_test_support::str;
-use cargo_test_support::{basic_manifest, cargo_process, project, rustc_host, Project};
+use cargo_test_support::{Project, basic_manifest, project, rustc_host};
 
 /// Switches Cargo.toml to use `resolver = "2"`.
 pub fn switch_to_resolver_2(p: &Project) {
@@ -263,7 +265,7 @@ common
 #[cargo_test]
 fn itarget_proc_macro() {
     // itarget inside a proc-macro while cross-compiling
-    if cross_compile::disabled() {
+    if cross_compile_disabled() {
         return;
     }
     Package::new("hostdep", "1.0.0").publish();
@@ -1124,10 +1126,11 @@ it is true
         .run();
     p.cargo("test --doc").env("TEST_EXPECTS_ENABLED", "1").run();
     p.cargo("doc").run();
-    assert!(p
-        .build_dir()
-        .join("doc/common/constant.FEAT_ONLY_CONST.html")
-        .exists());
+    assert!(
+        p.build_dir()
+            .join("doc/common/constant.FEAT_ONLY_CONST.html")
+            .exists()
+    );
     // cargo doc should clean in-between runs, but it doesn't, and leaves stale files.
     // https://github.com/rust-lang/cargo/issues/6783 (same for removed items)
     p.build_dir().join("doc").rm_rf();
@@ -1143,10 +1146,11 @@ it is false
 
     p.cargo("test --doc").run();
     p.cargo("doc").run();
-    assert!(!p
-        .build_dir()
-        .join("doc/common/constant.FEAT_ONLY_CONST.html")
-        .exists());
+    assert!(
+        !p.build_dir()
+            .join("doc/common/constant.FEAT_ONLY_CONST.html")
+            .exists()
+    );
 }
 
 #[cargo_test]
@@ -1312,7 +1316,7 @@ fn has_dev_dep_for_test() {
 #[cargo_test]
 fn build_dep_activated() {
     // Build dependencies always match the host for [target.*.build-dependencies].
-    if cross_compile::disabled() {
+    if cross_compile_disabled() {
         return;
     }
     Package::new("somedep", "1.0.0")
